@@ -1,9 +1,9 @@
-### dist.topo.R  (2005-05-31)
+### dist.topo.R  (2005-08-15)
 ###
 ###     Topological Distances, Tree Bipartition, and
 ###              Bootstrapping Phylogenies
 ###
-### Copyright 2005 Emmanuel Paradis <paradis@isem.univ-montp2.fr>
+### Copyright 2005 Emmanuel Paradis
 ###
 ### This file is part of the `ape' library for R and related languages.
 ### It is made available under the terms of the GNU General Public
@@ -21,36 +21,59 @@
 ### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 ### MA 02111-1307, USA
 
-dist.topo <- function(x, y)
+dist.topo <- function(x, y, method = "PH85")
 {
+    if (method == "BHV01" && (is.null(x$edge.length) || is.null(y$edge.length)))
+      stop("trees must have branch lengths for Billera et al.'s distance.")
     bp1 <- .Call("bipartition", as.integer(x$edge[, 1]),
                  as.integer(x$edge[, 2]), PACKAGE = "ape")
-    ## we remove the 2 first elements of 'bp1' since:
-    ##  - the 1st one contains all tips (maybe need to be changed!)
-    ##  - the 2nd one is redundant with the other node linked to the root
-    bp1 <- lapply(bp1[-(1:2)], function(xx) sort(x$tip.label[xx]))
+    bp1 <- lapply(bp1, function(xx) sort(x$tip.label[xx]))
     bp2 <- .Call("bipartition", as.integer(y$edge[, 1]),
                  as.integer(y$edge[, 2]), PACKAGE = "ape")
-    bp2 <- lapply(bp2[-(1:2)], function(xx) sort(y$tip.label[xx]))
+    bp2 <- lapply(bp2, function(xx) sort(y$tip.label[xx]))
     q1 <- length(bp1)
     q2 <- length(bp2)
     p <- 0
-    for (i in 1:length(bp1)) {
-        for (j in 1:length(bp2)) {
-            if (identical(all.equal(bp1[[i]], bp2[[j]]), TRUE)) {
-                p <- p + 1
-                break
+    if (method == "PH85") {
+        for (i in 1:q1) {
+            for (j in 1:q2) {
+                if (identical(all.equal(bp1[[i]], bp2[[j]]), TRUE)) {
+                    p <- p + 1
+                    break
+                }
             }
         }
+        dT <- if (q1 == q2) 2 * (q1 - p) else 2 * (min(q1, q2) - p) + abs(q1 - q2)
+    }
+    if (method == "BHV01") {
+        dT <- 0
+        found1 <- FALSE
+        found2 <- logical(q2)
+        for (i in 1:q1) {
+            for (j in 1:q2) {
+                if (identical(all.equal(bp1[[i]], bp2[[j]]), TRUE)) {
+                    if (i != 1 || j != 1)
+                      dT <- dT + x$edge.length[which(as.numeric(x$edge[, 2]) == -i)] -
+                               y$edge.length[which(as.numeric(x$edge[, 2]) == -j)]
+                    found1 <- found2[j] <- TRUE
+                    break
+                }
+            }
+            if (found1) {
+                found1 <- FALSE
+                next
+            } else dT <- dT + x$edge.length[which(as.numeric(x$edge[, 2]) == -j)]
+        }
+        if (any(!found2))
+          dT <- dT + sum(y$edge.length[as.numeric(y$edge[, 2]) %in% -which(!found2)])
     }
-    dT <- if (q1 == q2) 2 * (q1 - p) else 2 * (min(q1, q2) - p) + abs(q1 - q2)
     dT
 }
 
 prop.part <- function(...)
 {
     obj <- list(...)
-    if (length(obj) == 1 & class(obj[[1]]) != "phylo")
+    if (length(obj) == 1 && class(obj[[1]]) != "phylo")
       obj <- unlist(obj, recursive = FALSE)
     ntree <- length(obj)
     bp <- .Call("bipartition", as.integer(obj[[1]]$edge[, 1]),
@@ -87,7 +110,7 @@ prop.clades <- function(phy, ..., part = NULL)
 {
     if (is.null(part)) {
         obj <- list(...)
-        if (length(obj) == 1 & class(obj[[1]]) != "phylo")
+        if (length(obj) == 1 && class(obj[[1]]) != "phylo")
           obj <- unlist(obj, recursive = FALSE)
         part <- prop.part(obj)
     }
